% Author: Aparna Rajpurkar

function positions = GMMSegment(binimgs, origimgs)
    % GMM Segmentation algorithm

    % set seed
    rng(1);
    % initialize
    positions = cell([size(binimgs)]);

    % iterate over all binary images 
    for i=1:size(binimgs,1)
       % get the matching grey level image
       curr_merfish = imgaussfilt(origimgs{i});

       for j = 1:size(binimgs,2)
          % get the current binary image
          curr_binimg = binimgs{i,j};

          % get statistics on the connected components
          [coords, regions] = getCoordsOfComponents(curr_binimg);
          
          % initialize a structure to hold the centroids
          cents = zeros(size(curr_binimg));
          
          % if there aren't any connected components, go to next loop
          if (max(regions) == 0)
             continue; 
          end
          
          % iterate over all connected components
          for c=1:max(regions(:))
             % get the coordinates corresponding to this CC
             mask_region = regions == c;
             % find how many pixels are in this CC
             pixels_in_mask = sum(mask_region(:));
             
             % minimum size of component
             % so that we don't try to fit a GMM to something too small
             if (pixels_in_mask < 20)
                % if the CC is too small, assume it only has 1 spot
                % and set that in the centroid matrix and continue
                centroid_struct = regionprops(mask_region, 'centroid');
                centroid = centroid_struct.Centroid;
                centroid = uint8(round(centroid));
                centroid(centroid == 0) = 1;
                
                x = centroid(1);
                y = centroid(2);
                
                cents(x,y) = 1;
                
                continue;
             end
             
             % mask the grey level image with this CC
             mask_grey = curr_merfish.*mask_region;
             
             % get all the grey intensity values
             greypix = mask_grey(:);

             % find the best threshold dividing "dark" pixels
             % from "light" pixels, excluding black regions
             thresh = graythresh(greypix(greypix > 0));
             
             % find all pixels which are determined to be "light"
             [pos_x, pos_y] = find(mask_grey > thresh);
             coords = [pos_x pos_y];
             
             % set a maximum k value for the GMM
             % named "init_k" but this is the max k value
             % which will be 1/10 the area of the CC
             init_k = size(coords,1) / 10;
             init_k = round(init_k);
             
             % threshold k at 75 just for performance
             % otherwise it blows up my computer and doesn't finish
             % for a very long time
             % future directions: split up big CCs optimally
             % to remove the need for this
             if (init_k > 75)
                init_k = 75; 
             end
            
             % initialize array of AICs and GMMs 
             aics = zeros([init_k 1]);
             gmms = cell([init_k 1]);
             
             % calculate the GMM for every value of k < init_k
             for k=1:init_k
                gmms{k} = fitgmdist(coords,k, 'SharedCov', false, 'CovType','full', 'Regularize', 0.01);
                aics(k) = gmms{k}.AIC;
             end
             
             % find the GMM with the minimum AIC
             [minAIC, minIdx] = min(aics);
            
             % if for whatever reason we couldn't fit any GMMs, assume this region has 
             % only one spot and continue in the loop
             if (length(gmms) < 1)
                centroid_struct = regionprops(mask_region, 'centroid');
                centroid = centroid_struct.Centroid;
                centroid = uint8(round(centroid));
                centroid(centroid == 0) = 1;
                
                x = centroid(1);
                y = centroid(2);
                
                cents(x,y) = 1;
                
                continue;
             end
             
             % get the minGMM
             minGMM = gmms{minIdx};
             
             mu = minGMM.mu;
             sigma = minGMM.Sigma;
             p = minGMM.PComponents;
             
             % get the cluster labels
             [clustInd,~,p] = cluster(minGMM,coords);
             
             % for each cluster label, check if it is a
             % connected component, if so find the centroid
             % and add to the binary image
             for clust=1:max(clustInd)
                 rows = find(clustInd == clust);
                 curr_clust_coords = coords(rows,:);
                 temp_bin = zeros([size(curr_binimg)]);
                 
                 for n=1:length(rows)
                     x = curr_clust_coords(n,1);
                     y = curr_clust_coords(n,2);
                     temp_bin(x,y) = 1;
                 end
                
                 % get connected components 
                 temp_regions = regionprops(temp_bin);
                 
                 % if there aren't any, something's wrong just go to
                 % the next loop
                 if (length(temp_regions) == 0)
                    continue; 
                 end
                 
                 cent = zeros([1 2]);
                 
                 % if there's more than one connected component
                 % take the one with the maximum area, discard the rest
                 if (length(temp_regions) > 1)
                     [maxArea, maxIdx] = max(temp_regions(:).Area);
                     centroid = temp_regions(maxIdx).Centroid;
                     cent(1) = uint8(round(centroid(1)));
                     cent(2) = uint8(round(centroid(2)));
                 else
                     centroid = temp_regions(1).Centroid;
                     cent(1) = uint8(round(centroid(1)));
                     cent(2) = uint8(round(centroid(2)));
                 end
                 cent(cent == 0) = 1;
                 cents(cent(1), cent(2)) = 1; 
             end
          end
          % set the centroid in the binary image
          positions{i,j} = cents;
       end
    end
end
