% Author: Aparna Rajpurkar

function [prs, random] = comparePerformance(binimgs, truth, search_window)
    % compare the performance of the algorithms by using precision recall plots

    % initialize
    prs = zeros([size(binimgs,1) size(binimgs,2) 2]);
    random = zeros([size(binimgs,1) 1]);
    
    % loop over all images
    for i=1:size(binimgs,1)
       curr_truth = truth{i};
       
       % initialize
       pos = length(find(curr_truth==1));
       neg = length(find(curr_truth==0));

       % calculate the randomness threshold
       random(i) = pos / (pos + neg);
       
       for j=1:size(binimgs,2)
           curr_bin = binimgs{i,j};

           % calculate TP, FP, TN, FN rates
           [TP,FP] = getTPFP(curr_bin, curr_truth, search_window);
           [TN,FN] = getTNFN(curr_bin, curr_truth, search_window);
           
           % calculate precision and recall
           precision = TP / (TP + FP);
           recall = TP / (TP + FN);
           
           % check if TP and FP are zero
           if ((TP + FP) == 0)
              precision = 0; 
           end
           
           % check if TN and FN are zero
           if ((TN + FN) == 0)
              recall = 0; 
           end
           
           % add values to data structure
           prs(i,j,1) = precision;
           prs(i,j,2) = recall;
       end
    end
end

function [TP,FP] = getTPFP(curr_bin, truth, win)
    % calculate the TP and FP rate by looking for a ground truth
    % centroid within a window of 5 px.
    [y_coords, x_coords] = find(curr_bin == 1);
    
    num_pairs = length(x_coords);
    x_win = size(truth,1);
    y_win = size(truth,2);
    
    TP = 0;
    FP = 0;
    for i=1:num_pairs
       x = x_coords(i);
       y = y_coords(i);
       
       max_x = min(x + win, x_win);
       max_y = min(y + win, y_win);
       min_x = max(1, x - win);
       min_y = max(1, y - win);
       
       searchbox = truth(min_x:max_x, min_y:max_y);
       if (sum(searchbox(:)) > 0)
          TP = TP + 1;
       else
           FP = FP + 1;
       end
    end
end

function [TN,FN] = getTNFN(curr_bin, truth, win)
    % calculate the TN and FN rate by looking for a ground truth
    % centroid within a window of 5 px.
    [y_coords, x_coords] = find(curr_bin == 0);
    
    num_pairs = length(x_coords);
    x_win = size(truth,1);
    y_win = size(truth,2);
    
    TN = 0;
    FN = 0;
    
    for i=1:num_pairs
       x = x_coords(i);
       y = y_coords(i);
       
       max_x = min(x + win, x_win);
       max_y = min(y + win, y_win);
       min_x = max(1, x - win);
       min_y = max(1, y - win);
       
       searchbox = truth(min_x:max_x, min_y:max_y);
       if (sum(searchbox(:)) > 0)
          FN = FN + 1;
       else
          TN = TN + 1;
       end
    end
end
