% Author: Aparna Rajpurkar
% Master driver script to do all the analysis for the final project

%% MERFISH data

DIR_ALIGNED_MERFISH = '~/merfish_run_all/aligned_imgs/';
DIR_GROUND_TRUTH = '~/merfish_run_all/ground_truth/';
FUNCS_PATH = '~/merfish_run_all/funcs/';

% add functions to matlab path
addpath(FUNCS_PATH);

% make a list of core test functions
funcs_list = {@AdaptiveThreshold, @KMeansSegment, @GraphCutSegment, @MSERDetect};

% make a list of ensemble functions
ensemble_list = {@IntersectResults, @UnionResults, @VoteResults};

% make a list of all functions 
allfuncs = [funcs_list ensemble_list];

% make a list of function names
func_names = cell([length(allfuncs) 1]);

for i=1:length(allfuncs)
    func_names{i} = func2str(allfuncs{i});
end

func_names = char(func_names);

%%

% read in the MERFISH files
[merfish, truth] = readMerfish(DIR_ALIGNED_MERFISH, DIR_GROUND_TRUTH);

% run algorithms on MERFISH files
[binimgs, fullbins] = runFuncsOnImgs(merfish, funcs_list, ensemble_list);

%%
% run GMM based segmentation method
[segmentResult] = GMMSegment(fullbins, merfish);
%%
% compute accuracy of all algorithms on all images by precision recall
SEARCH_WIN = 5;
[prs,random] = comparePerformance(binimgs, truth, SEARCH_WIN);
[prs_segment,random_segment] = comparePerformance(segmentResult, truth, SEARCH_WIN);

%%
% plot accuracy
plotPrs(prs, random, func_names, 'NoSegment');
plotPrsCombined(prs, prs_segment, random, func_names, 'Combined');

%%
% plot binary centroid masks on original images
for i=1:length(truth)
    plotBinaryImageOnImg(truth{i}, merfish{i}, strcat('GroundTruth_',num2str(i)));
end

for i=1:size(binimgs,2)
    plotBinaryImageOnImg(binimgs{1,i}, merfish{1}, strcat('alg_',num2str(i),'.png'));
end

for i=1:size(binimgs,2)
    plotBinaryImageOnImg(segmentResult{1,i}, merfish{1}, strcat('segalg_',num2str(i),'.png'));
end

%% Mouse dataset

% data type: jpgs 
% 2 treated and 2 untreated samples

% globals
DAPI_PATTERN = 'MAX_CH1_DAPI.jpg';
CENP_PATTERN = 'MAX_CH2_CenpB.jpg';
TELC_PATTERN = 'MAX_CH3_TelC.jpg';

TREATED1_DIR = '~/Documents/MATLAB/CS232/mouse_images/treated1/';
TREATED2_DIR = '~/Documents/MATLAB/CS232/mouse_images/treated2/';
UNTREATED1_DIR = '~/Documents/MATLAB/CS232/mouse_images/untreated1/';
UNTREATED2_DIR = '~/Documents/MATLAB/CS232/mouse_images/untreated2/';

% get DAPI stain for nuclei segmentation
[mouse_treated_dapi] = readMouse(TREATED1_DIR, TREATED2_DIR, DAPI_PATTERN);
[mouse_untreated_dapi] = readMouse(UNTREATED1_DIR, UNTREATED2_DIR, DAPI_PATTERN);

% get centromere data
[mouse_treated_cenp] = readMouse(TREATED1_DIR, TREATED2_DIR, CENP_PATTERN);
[mouse_untreated_cenp] = readMouse(UNTREATED1_DIR, UNTREATED2_DIR, CENP_PATTERN);

% get telomere data
[mouse_treated_telc] = readMouse(TREATED1_DIR, TREATED2_DIR, TELC_PATTERN);
[mouse_untreated_telc] = readMouse(UNTREATED1_DIR, UNTREATED2_DIR, TELC_PATTERN);

%%
% segment telc and cenp cells by DAPI stain
[cenp_per_cell_treated] = DapiSegment(mouse_treated_dapi, mouse_treated_cenp);
[cenp_per_cell_untreated] = DapiSegment(mouse_untreated_dapi, mouse_untreated_cenp);

[telc_per_cell_treated] = DapiSegment(mouse_treated_dapi, mouse_treated_telc);
[telc_per_cell_untreated] = DapiSegment(mouse_untreated_dapi, mouse_untreated_telc);

%%
% run funcs on mouse cells
[binimgs_cenp_treated, fullbins_cenp_treated] = runFuncsOnImgs(cenp_per_cell_treated, funcs_list, {});
[binimgs_cenp_untreated, fullbins_cenp_untreated] = runFuncsOnImgs(cenp_per_cell_untreated, funcs_list, {});
[binimgs_telc_treated, fullbins_telc_treated] = runFuncsOnImgs(telc_per_cell_treated, funcs_list, {});
[binimgs_telc_untreated, fullbins_telc_untreated] = runFuncsOnImgs(telc_per_cell_untreated, funcs_list, {});

%% 
% run segmentation
[segmentResult_cenp_treated] = GMMSegment(fullbins_cenp_treated, cenp_per_cell_treated);
[segmentResult_cenp_untreated] = GMMSegment(fullbins_cenp_untreated, cenp_per_cell_untreated);
[segmentResult_telc_treated] = GMMSegment(fullbins_telc_treated, telc_per_cell_treated);
[segmentResult_telc_untreated] = GMMSegment(fullbins_telc_untreated, telc_per_cell_untreated);


%%
% plot example binary images on original image
for i=1:size(segmentResult_cenp_treated,2)
    plotBinaryImageOnImg(segmentResult_cenp_treated{1,i}, cenp_per_cell_treated{1}, strcat('segment_cenp_example_treated_',num2str(i),'.png'));
end

for i=1:size(segmentResult_telc_treated,2)
    plotBinaryImageOnImg(segmentResult_telc_treated{3,i}, telc_per_cell_treated{3}, strcat('segment_telc_example_treated_',num2str(i),'.png'));
end

%%
% get counts

[cenp_treated_counts] = getCountsFromBinimgs(segmentResult_cenp_treated);
[cenp_untreated_counts] = getCountsFromBinimgs(segmentResult_cenp_untreated);

[telc_treated_counts] = getCountsFromBinimgs(segmentResult_telc_treated);
[telc_untreated_counts] = getCountsFromBinimgs(segmentResult_telc_untreated);

%% 
% make boxplots

makeBoxplot(cenp_treated_counts, 'Centromere Counts: Treated Sample', 'centromere counts', func_names(1:length(funcs_list),:), 'seg_cenp_treated_box')
makeBoxplot(cenp_untreated_counts, 'Centromere Counts: Untreated Sample', 'centromere counts', func_names(1:length(funcs_list),:), 'seg_cenp_untreated_box')

makeBoxplot(telc_treated_counts, 'Telomere Counts: Treated Sample', 'telomere counts', func_names(1:length(funcs_list),:), 'seg_telc_treated_box')
makeBoxplot(telc_untreated_counts, 'Telomere Counts: Untreated Sample', 'telomere counts', func_names(1:length(funcs_list),:), 'seg_telc_untreated_box')

